import logging

from heaobject.root import Permission, ShareImpl
from heaobject.data import DataObject, SameMimeType
from collections.abc import Iterable, Iterator
from typing import Optional

permission_id_dict = {
    'manager_ids': [Permission.VIEWER, Permission.CREATOR, Permission.EDITOR, Permission.SHARER, Permission.DELETER],
    'member_ids': [Permission.VIEWER, Permission.EDITOR, Permission.SHARER]
}


class Organization(DataObject, SameMimeType):
    """
    Represents a directory in the HEA desktop.
    """

    def __init__(self):
        super().__init__()
        # id is a super field
        self.__aws_account_ids: list[str] = []  # allow org to have multiple aws accounts
        self.__principal_investigator_id: Optional[str] = None  # this would be a people id
        self.__manager_ids: list[str] = []  # list of user ids to be managers
        self.__member_ids: list[str] = []  # list of user ids to be members
        # super's name and display name would be used as org name(required)

    @classmethod
    def get_mime_type(cls) -> str:
        """
        Returns the mime type of instances of the Organization class.

        :return: application/x.organization
        """
        return 'application/x.organization'

    @property
    def mime_type(self) -> str:
        """Read-only. The mime type for Organization objects, application/x.organization."""
        return type(self).get_mime_type()

    @property
    def aws_account_ids(self) -> list[str]:
        """
        The list of REST aws account ids that are served by this organization. The property's setter accepts a list of
        strings.
        """
        return [i for i in self.__aws_account_ids]

    @aws_account_ids.setter
    def aws_account_ids(self, value: list[str]) -> None:
        if value is None:
            self.__aws_account_ids = []
        elif isinstance(value, str) or not isinstance(value, Iterable | Iterator):
            self.__aws_account_ids = [value]
        else:
            self.__aws_account_ids = [str(i) for i in value]

    def add_aws_account_id(self, value: str) -> None:
        """
        Adds a REST resource to the list of resources that are served by this component.
        :param value: a Resource object.
        """

        self.__aws_account_ids.append(str(value))

    def remove_aws_account_id(self, value: str) -> None:
        """
        Removes a REST aws_account_id from the list of ids that are served by this organization. Ignores None values.
        :param value: str representing the aws account id.
        """

        self.__aws_account_ids.remove(str(value))

    @property
    def principal_investigator_id(self) -> Optional[str]:
        """
        The principal investigator People ID.
        """
        return self.__principal_investigator_id

    @principal_investigator_id.setter
    def principal_investigator_id(self, principal_investigator_id: Optional[str]) -> None:
        self.__principal_investigator_id = str(principal_investigator_id) \
            if principal_investigator_id is not None else None

    @property
    def manager_ids(self) -> list[str]:
        """
        The organization manager ids.
        """
        return [i for i in self.__manager_ids] if self.__manager_ids else []

    @manager_ids.setter
    def manager_ids(self, manager_ids: list[str]) -> None:
        if manager_ids is None:
            self.__manager_ids = []
        elif isinstance(manager_ids, str) or not isinstance(manager_ids, Iterable | Iterator):
            self.__manager_ids = [manager_ids]
        else:
            self.__manager_ids = [str(i) for i in manager_ids]

    def add_manager_id(self, value: str) -> None:
        self.__manager_ids.append(str(value))

    def remove_manager_id(self, value: str) -> None:
        """
        Removes a REST manager id from the list of ids that are served by this organization. Ignores None values.
        :param value:  str representing the manager id.
        """
        self.__manager_ids.remove(str(value))

    @property
    def member_ids(self) -> list[str]:
        """
        The organization member ids.
        """
        return [i for i in self.__member_ids]

    @member_ids.setter
    def member_ids(self, member_ids: list[str]) -> None:
        if member_ids is None:
            self.__member_ids = []
        elif isinstance(member_ids, str) or not isinstance(member_ids, Iterable | Iterator):
            self.__member_ids = [member_ids]
        else:
            self.__member_ids = [str(i) for i in member_ids]

    def add_member_id(self, value: str) -> None:
        self.__member_ids.append(str(value))

    def remove_member_id(self, value: str) -> None:
        """
        Removes a REST member id from the list of member ids that are served by this organization. Ignores None values.
        :param value: a str representing the member id.
        """
        self.__member_ids.remove(str(value))

    def dynamic_permission(self, sub: str) -> list[Permission]:
        """
        Returns VIEWER, EDITOR, and SHARER permissions if the sub is in the member_ids list, or an empty list if not.

        :param sub: the user id (required).
        :return: A list containing Permissions or the empty list.
        """
        if not isinstance(sub, str):
            logging.warning(f'sub is not a str (got {type(sub).__name__}), so dynamic permissions may fail')
        sub_ = str(sub)
        try:
            perms: list[Permission] = []
            for p_id in permission_id_dict:
                if sub_ in getattr(self, p_id):
                    # Union of two lists
                    perms = list(set(permission_id_dict[p_id]) | set(perms))
        except Exception as e:
            logging.error('Permissions are not correctly configured...returning empty permissions set')
            logging.error(e)
            return []

        return perms



"""
Defines the following reserved usernames:

NONE_USER: 'system|none' -- for files that are installed by default and are not owned by any particular user.
ALL_USERS: 'system|all' -- for use in sharing a file with all users.
TEST_USER: 'system|test' -- for use in testing HEA.

Furthermore, all usernames beginning with 'system|' are reserved for system use.
"""

NONE_USER = 'system|none'
ALL_USERS = 'system|all'
TEST_USER = 'system|test'



from enum import auto
from typing import Optional, Union
from .data import DataObject, SameMimeType
from abc import ABC

from .root import EnumAutoName


class Status(EnumAutoName):
    """
    The standard permissions that apply to all HEA desktop objects.
    """
    COMPLETE = auto()  # No access to the object unless dynamic_permission() returns a non-None value.
    FAILED = auto()
    IN_PROGRESS = auto()


class Activity(DataObject, ABC):
    """
       Abstract base class for user activities.
       """
    pass


class UserActivity(Activity, SameMimeType):
    """
        Represents an user activity in the HEA desktop. Contains functions that allow access and setting of the value.

        activity_id (str)             : 123456
        user_id     (str)             : 123456
        action      (str)             : update
        status      (str)             : FAILED
        arn         (str)             : my_bucket
        """

    def __init__(self):
        super().__init__()
        self.__user_id: Optional[str] = None
        self.__action: Optional[str] = None
        self.__status: Optional[Union[Status, str]] = None
        self.__arn: Optional[str] = None

    @classmethod
    def get_mime_type(cls) -> str:
        return 'application/x.activity'

    @property
    def mime_type(self) -> str:
        return type(self).get_mime_type()

    @property
    def user_id(self) -> Optional[str]:
        """Returns the numerical user identifier"""
        return self.__user_id

    @user_id.setter
    def user_id(self, __user_id: Optional[str]) -> None:
        """Sets the numerical user identifier"""
        self.__user_id = str(__user_id) if __user_id is not None else None

    @property
    def action(self) -> Optional[str]:
        """Returns the string describes user action"""
        return self.__action

    @action.setter
    def action(self, __action: Optional[str]) -> None:
        """Sets the string describes user action"""
        self.__action = str(__action) if __action is not None else None

    @property
    def status(self) -> Optional[Union[Status, str]]:
        """Returns the string describes actions status"""
        return self.__status

    @status.setter
    def status(self, __status: Optional[Union[Status, str]]) -> None:
        """Sets the string describes actions status"""
        try:
            if isinstance(__status, Status):
                self.__status = __status
            elif isinstance(__status, str):
                self.__status = Status(__status)
            elif __status is None:
                self.__status = __status
            else:
                raise Exception()
        except Exception as e:
            raise KeyError(
                "Status can only be a Status object or string that can be converted to Status object")

    @property
    def arn(self) -> Optional[str]:
        """Returns the AWS resource identifier"""
        return self.__arn

    @arn.setter
    def arn(self, __arn: Optional[str]) -> None:
        """Sets the AWS resource identifier"""
        self.__arn = str(__arn) if __arn is not None else None



"""
Defines volume and file system desktop object classes.

Volumes and file systems are defined analogously to operating system volumes (sometimes called mounts or drives) and
file systems. File systems are repositories of HEA objects. They may be actual file storage on the server running HEA,
a database, a web service, or a separate file server accessed via standard network protocols like SSH. Volumes provide
a connection to that database or service, possibly passing in credentials. Volumes and file systems are represented by
the Volume and FileSystem classes in this module, respectively. Volumes have a file_system_name attribute with a name
of a FileSystem instance.

Microservices that manage data or application state have a default data store, typically MongoDB, with its connection
information specified in a configuration file. To use the default data store for managing HEA desktop objects, if the
microservice supports it, create a volume with the file_system_name set to DEFAULT_FILE_SYSTEM or None. Or use variants
of the microservice's REST API calls that do not require passing in a volume id.

"""
from abc import ABC
from typing import Optional, TypeVar
from . import root
from .data import DataObject, SameMimeType


class FileSystem(root.AbstractDesktopObject, ABC):
    """
    Represents a filesystem, which controls how data is stored and retrieved. In HEA, all filesystems are either
    databases or network storage.

    File systems must have a name property that is unique per concrete FileSystem subclass. The name is typically a
    connection string, account id, or similar field.

    A special subclass of FileSystem, DefaultFileSystem, represents the default storage of a microservice, typically
    a MongoDB database, but possibly some other data store.
    """
    pass


FileSystemTypeVar = TypeVar('FileSystemTypeVar', bound=FileSystem)


class DefaultFileSystem(FileSystem):
    """
    The default file system. It is a special subclass of FileSystem with a read-only name property that always returns
    the DEFAULT_FILE_SYSTEM constant.
    """
    def __init__(self):
        super().__init__()

    def __get_name(self) -> str:
        return DEFAULT_FILE_SYSTEM

    name = property(__get_name, doc='Always returns DEFAULT_FILE_SYSTEM. Is read-only.')  # type: ignore


class MongoDBFileSystem(FileSystem):
    """
    MongoDB-based file system.
    """

    def __init__(self):
        super().__init__()
        self.__connection_string: Optional[str] = None
        self.__database_name: Optional[str] = None

    @property  # type: ignore
    def connection_string(self) -> Optional[str]:
        """The MongoDB connection string."""
        return self.__connection_string

    @connection_string.setter
    def connection_string(self, connection_string: Optional[str]):
        self.__connection_string = str(connection_string) if connection_string is not None else None

    @property  # type: ignore
    def database_name(self) -> Optional[str]:
        """The MongoDB database name"""
        return self.__database_name

    @database_name.setter
    def database_name(self, database_name: Optional[str]):
        self.__database_name = str(database_name) if database_name is not None else None


class AWSFileSystem(FileSystem):
    """
    AWS-based file system. There can be at most one global AWS file system with name DEFAULT_FILE_SYSTEM, and users may
    have one volume for accessing their AWS account, with an association to a credentials object with the user's AWS
    key and secret.
    """
    def __init__(self):
        super().__init__()

    name = property(fget=lambda self: 'DEFAULT_FILE_SYSTEM', doc="Always returns DEFAULT_FILE_SYSTEM. Read-only.")  # type: ignore


class Volume(DataObject, SameMimeType):
    """
    A single accessible storage area that stores a single filesystem. Some volumes may require providing credentials in
    order to access them.
    """

    @classmethod
    def get_mime_type(cls) -> str:
        return 'application/x.volume'

    def __init__(self):
        super().__init__()
        self.__file_system_name = DEFAULT_FILE_SYSTEM
        self.__file_system_type = DefaultFileSystem.get_type_name()
        self.__credential_id: Optional[str] = None
        self.__folder_id: Optional[str] = None

    @property
    def mime_type(self) -> str:
        return type(self).get_mime_type()

    @property  # type: ignore
    def folder_id(self) -> Optional[str]:
        """
        The id of the folder to open when opening this volume.
        """
        return self.__folder_id

    @folder_id.setter
    def folder_id(self, id_) -> None:
        self.__folder_id = str(id_) if id_ is not None else None

    @property  # type: ignore
    def file_system_name(self) -> str:
        """
        The unique name of this volume's file system (a FileSystem object). Defaults to the 'root' file system
        (DEFAULT_FILE_SYSTEM). Has the same value as the type property.
        """
        return self.__file_system_name

    @file_system_name.setter  # type: ignore
    def file_system_name(self, file_system_name: str) -> None:
        self.__file_system_name = str(file_system_name) if file_system_name is not None else DEFAULT_FILE_SYSTEM

    @property  # type: ignore
    def file_system_type(self) -> str:
        """
        The type name of this volume's file system (a FileSystem object). Defaults to the 'root' file system
        (heaobject.volume.DefaultFileSystem). Has the same value as the name property.
        """
        return self.__file_system_type

    @file_system_type.setter  # type: ignore
    def file_system_type(self, file_system_type: str) -> None:
        self.__file_system_type = str(file_system_type) if file_system_type is not None else DefaultFileSystem.get_type_name()

    def __get_name(self) -> str:
        """
        Gets the file system name.
        :return: the file system name.
        """
        return f'{self.file_system_type}^{self.file_system_name}'

    name = property(__get_name, doc="The unique name of this volume's file system (a FileSystem object). It is a concatenation of the file system type and name. Read-only.")  # type: ignore

    @property  # type: ignore
    def credential_id(self) -> Optional[str]:
        """
        The id of the current user's credentials needed to open this volume (a Credentials object), if any.
        """
        return self.__credential_id

    @credential_id.setter  # type: ignore
    def credential_id(self, credentials_id: Optional[str]) -> None:
        self.__credential_id = str(credentials_id) if credentials_id is not None else None


DEFAULT_FILE_SYSTEM = 'DEFAULT_FILE_SYSTEM'



from typing import Optional
from .data import DataObject, SameMimeType
from abc import ABC
from email_validator import validate_email, EmailNotValidError  # Leave this here for other modules to use
from functools import partial


class Account(DataObject, ABC):
    """
    Abstract base class for user accounts.
    """
    pass


class AWSAccount(Account, SameMimeType):
    """
    Represents an AWS account in the HEA desktop. Contains functions that allow access and setting of the value. Below are the attributes that can be accessed.

    account_id (str)              : 1234567890
    account_name (str)            : HCI - name
    full_name (str)               : john smith
    phone_number (str)            : 123-456-7890
    alternate_contact_name (str)  : bob smith
    alternate_email_address (str) : 123@hciutah.edu
    alternate_phone_number (str)  : 123-456-7890
    """
    def __init__(self):
        super().__init__()
        self.__full_name: Optional[str] = None
        self.__phone_number: Optional[str] = None
        self.__alternate_contact_name: Optional[str] = None
        self.__alternate_email_address: Optional[str] = None
        self.__alternate_phone_number: Optional[str] = None
        self.__email_address: Optional[str] = None

    @classmethod
    def get_mime_type(cls) -> str:
        """
        Returns the mime type for AWSAccount objects.

        :return: application/x.awsaccount
        """
        return 'application/x.awsaccount'

    @property
    def mime_type(self) -> str:
        """Read-only. The mime type for AWSAccount objects, application/x.awsaccount."""
        return type(self).get_mime_type()


    @property
    def full_name(self) -> Optional[str]:
        """Returns the full name of person associated with this account"""
        return self.__full_name

    @full_name.setter
    def full_name(self, full_name: Optional[str]) -> None:
        """Sets the full name of person associated with this account"""
        self.__full_name = str(full_name) if full_name is not None else None

    @property
    def phone_number(self) -> Optional[str]:
        """Returns the phone number associated with the account"""
        return self.__phone_number

    @phone_number.setter
    def phone_number(self, phone_number: Optional[str]) -> None:
        """Sets the phone number associated with the account"""
        self.__phone_number = str(phone_number) if phone_number is not None else None

    @property
    def alternate_contact_name(self) -> Optional[str]:
        """Returns the alternate contact full name of person associated with this account"""
        return self.__alternate_contact_name

    @alternate_contact_name.setter
    def alternate_contact_name(self, alternate_contact_name: Optional[str]) -> None:
        """Sets the alternate contact full name of person associated with this account"""
        self.__alternate_contact_name = str(alternate_contact_name) if alternate_contact_name is not None else None

    @property
    def alternate_email_address(self) -> Optional[str]:
        """Returns the alternate contact e-mail address associated with the account"""
        return self.__alternate_email_address

    @alternate_email_address.setter
    def alternate_email_address(self, alternate_email_address: Optional[str]) -> None:
        """Sets the alternate contact e-mail address associated with the account"""
        self.__alternate_email_address = _validate_email(str(alternate_email_address)).email \
            if alternate_email_address is not None else None

    @property
    def alternate_phone_number(self) -> Optional[str]:
        """Returns the alternate contact phone number associated with the account"""
        return self.__alternate_phone_number

    @alternate_phone_number.setter
    def alternate_phone_number(self, alternate_phone_number: Optional[str]) -> None:
        """Sets the alternate contact phone number associated with the account"""
        self.__alternate_phone_number = str(alternate_phone_number) if alternate_phone_number is not None else None

    @property
    def email_address(self) -> Optional[str]:
        return self.__email_address

    @email_address.setter
    def email_address(self, email_address: Optional[str]) -> None:
        """Sets the email address associated with the account"""
        self.__email_address = _validate_email(str(email_address)).email if email_address is not None else None


_validate_email = partial(validate_email, check_deliverability=False)



from .data import DataObject, SameMimeType
from .aws import S3URIMixin, S3StorageClassMixin
from typing import Optional
import copy
from humanize import naturalsize


class Folder(DataObject, SameMimeType):
    """
    Represents a directory in the HEA desktop.
    """
    def __init__(self):
        super().__init__()

    @classmethod
    def get_mime_type(cls) -> str:
        """
        Returns the mime type of instances of the Folder class.

        :return: application/x.folder
        """
        return 'application/x.folder'

    @property
    def mime_type(self) -> str:
        """
        Read-only. Always returns 'application/x.folder'.
        """
        return type(self).get_mime_type()


class AWSS3Folder(Folder, S3StorageClassMixin, S3URIMixin):
    """
    Represents folders stored in AWS S3.
    """
    pass


class Item(DataObject):
    """
    Represents an item in a folder. Items contain the actual object being pointed to as a nested
    """
    def __init__(self):
        super().__init__()
        self.__actual_object_type_name: Optional[str] = None
        self.__actual_object_id: Optional[str] = None
        self.__actual_object_uri: Optional[str] = None
        self.__folder_id: Optional[str] = None
        self.__volume_id: Optional[str] = None
        self.__size: Optional[int] = None

    @property
    def mime_type(self) -> str:
        """
        Read-only. Always returns 'application/x.item'.
        """
        return 'application/x.item'

    @property
    def actual_object_type_name(self) -> Optional[str]:
        """
        The name of the type of the actual HEAObject.
        """
        return self.__actual_object_type_name

    @actual_object_type_name.setter
    def actual_object_type_name(self, actual_object_type_name: Optional[str]) -> None:
        self.__actual_object_type_name = str(actual_object_type_name) if actual_object_type_name is not None else None

    @property
    def actual_object_id(self) -> Optional[str]:
        """
        The id of the actual HEAObject.
        """
        return self.__actual_object_id

    @actual_object_id.setter
    def actual_object_id(self, actual_object_id: Optional[str]) -> None:
        self.__actual_object_id = str(actual_object_id) if actual_object_id is not None else None

    @property
    def actual_object_uri(self) -> Optional[str]:
        """
        The actual HEAObject's URI.
        """
        return self.__actual_object_uri

    @actual_object_uri.setter
    def actual_object_uri(self, actual_object_uri: Optional[str]) -> None:
        self.__actual_object_uri = str(actual_object_uri) if actual_object_uri is not None else None

    @property
    def folder_id(self) -> Optional[str]:
        """
        The id of this item's folder.
        """
        return self.__folder_id

    @folder_id.setter
    def folder_id(self, folder_id: Optional[str]) -> None:
        self.__folder_id = str(folder_id) if folder_id is not None else None

    @property
    def volume_id(self) -> Optional[str]:
        """
        The id of this item's volume.
        """
        return self.__volume_id

    @volume_id.setter
    def volume_id(self, volume_id: Optional[str]) -> None:
        self.__volume_id = str(volume_id) if volume_id is not None else None

    @property
    def size(self) -> Optional[int]:
        """Size of the item in bytes"""
        return self.__size

    @size.setter
    def size(self, size: Optional[int]) -> None:
        self.__size = int(size) if size is not None else None

    @property
    def human_readable_size(self) -> str:
        return naturalsize(self.size) if self.size else None


class AWSS3Item(Item, S3StorageClassMixin, S3URIMixin):
    """
    Represents items stored in AWS S3.
    """
    pass



import json
from typing import Optional, Any

from ._util import str_to_bool
from .data import DataObject, SameMimeType
from abc import ABC

from .root import Tag
from .aws import S3URIMixin


class Bucket(DataObject, ABC):
    """
    Abstract base class for user accounts.
    """
    pass


class AWSBucket(Bucket, SameMimeType, S3URIMixin):
    """
    Represents an AWS Bucket in the HEA desktop. Contains functions that allow access and setting of the value.
    """

    def __init__(self):
        super().__init__()
        self.__arn: Optional[str] = None
        self.__encrypted: Optional[bool] = None
        self.__versioned: Optional[bool] = None
        self.__locked: Optional[bool] = None
        self.__region: Optional[str] = None
        self.__size: Optional[float] = None
        self.__object_count: Optional[int] = None
        self.__tags: list[Tag] = []
        self.__permission_policy: Optional[str] = None

    @classmethod
    def get_mime_type(cls) -> str:
        """
        Returns the mime type for AWSBucket objects.

        :return: application/x.awsbucket
        """
        return 'application/x.awsbucket'

    @property
    def mime_type(self) -> str:
        """Read-only. The mime type for AWSBucket objects, application/x.awsbucket."""
        return type(self).get_mime_type()

    @property
    def arn(self) -> Optional[str]:
        """Returns the aws arn str for identifying resources on aws"""
        return self.__arn

    @arn.setter
    def arn(self, arn: Optional[str]) -> None:
        """Sets the numerical account identifier"""
        self.__arn = str(arn) if arn is not None else None

    @property
    def encrypted(self) -> Optional[bool]:
        """Returns the is encrypted flag for bucket"""
        return self.__encrypted

    @encrypted.setter
    def encrypted(self, encrypted: Optional[bool]) -> None:
        """Sets the is encrypted flag for bucket"""
        if encrypted is None:
            self.__encrypted = None
        elif isinstance(encrypted, bool):
            self.__encrypted = encrypted
        else:
            self.__encrypted = str_to_bool(encrypted)  # type: ignore

    @property
    def versioned(self) -> Optional[bool]:
        """Returns the is versioned flag for bucket"""
        return self.__versioned

    @versioned.setter
    def versioned(self, versioned: Optional[bool]) -> None:
        """Sets the is versioned flag for bucket"""
        if versioned is None:
            self.__versioned = None
        elif isinstance(versioned, bool):
            self.__versioned = versioned
        else:
            self.__versioned = str_to_bool(versioned)  # type: ignore

    @property
    def locked(self) -> Optional[bool]:
        """Returns the  flag that objects are 'locked' for bucket"""
        return self.__locked

    @locked.setter
    def locked(self, locked: Optional[bool]) -> None:
        """Sets the flag that objects are 'locked'"""
        if locked is None:
            self.__locked = None
        elif isinstance(locked, bool):
            self.__locked = locked
        else:
            self.__locked = str_to_bool(locked)  # type: ignore

    @property
    def region(self) -> Optional[str]:
        """Returns the bucket region"""
        return self.__region

    @region.setter
    def region(self, region: Optional[str]) -> None:
        """Sets the bucket region"""
        self.__region = str(region) if region is not None else None

    @property
    def size(self) -> Optional[float]:
        """Returns the bucket size"""
        return self.__size

    @size.setter
    def size(self, size: float) -> None:
        """Sets the bucket size"""
        self.__size = float(size) if size is not None else None

    @property
    def object_count(self) -> Optional[int]:
        """Returns the number of objects in the bucket"""
        return self.__object_count

    @object_count.setter
    def object_count(self, object_count: int) -> None:
        """Sets the number of objects in the bucket"""
        self.__object_count = int(object_count) if object_count is not None else None

    @property
    def tags(self) -> list[Tag]:
        """Returns the bucket tags"""
        return self.__tags

    @tags.setter
    def tags(self, tags: list[Tag]) -> None:
        """Sets the bucket tags"""
        if tags is not None and type(tags) is not list:
            raise ValueError("This format is not correct for tags, type should be list")
        if not all(isinstance(tag, Tag) for tag in (tags or [])):
            raise ValueError("This format is not correct list must contain tags")
        self.__tags = tags if tags is not None else []

    @property
    def permission_policy(self) -> Optional[str]:
        """Returns the permission policy as string representation of json"""
        return self.__permission_policy

    @permission_policy.setter
    def permission_policy(self, permission_policy: Optional[str] = None) -> None:
        """Sets the permission policy as json str"""

        if permission_policy is not None and type(permission_policy) is not str:
            """testing if valid json str"""
            raise ValueError("not valid permission policy json, type should be str.")

        self.__permission_policy = permission_policy if permission_policy is not None else None



"""
Utility classes and functions for working with AWS.
"""
from enum import auto

from heaobject import root
from typing import Optional


class S3StorageClass(root.EnumAutoName):
    """
    The S3 storage classes. The list of storage classes is documented at
    https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html#S3.Client.list_objects_v2, and
    each storage class is explained in detail at
    https://docs.aws.amazon.com/AmazonS3/latest/userguide/storage-class-intro.html.
    """
    STANDARD = auto()  # S3 Standard
    REDUCED_REDUNDANCY = auto()  # Reduced Redundancy (RRS)
    GLACIER = auto()  # S3 Glacier Flexible Retrieval
    STANDARD_IA = auto()  # S3 Standard-IR (infrequent access)
    ONEZONE_IA = auto()  # S3 One Zone-IR
    INTELLIGENT_TIERING = auto()  # S3 Intelligent-Tiering
    DEEP_ARCHIVE = auto()  # S3 Glacier Deep Archive
    OUTPOSTS = auto()  # S3 Outposts
    GLACIER_IR = auto()  # S3 Glacier Instant Retrieval
    OTHER = auto()


class S3URIMixin:
    """
    Mixing for adding a S3 URI to a desktop object.
    """

    @property
    def s3_uri(self) -> Optional[str]:
        """
        The object's S3 URI.
        """
        try:
            return self.__s3_uri
        except AttributeError:
            return None

    @s3_uri.setter
    def s3_uri(self, s3_uri: Optional[str]):
        s3_uri_ = str(s3_uri) if s3_uri is not None else None
        if s3_uri_ is not None and not s3_uri_.startswith('s3://'):
            raise ValueError(f'Invalid s3 URI {s3_uri_}')
        self.__s3_uri = s3_uri_

    @property
    def presigned_url(self) -> Optional[str]:
        """
        The object's presigned url.
        """
        try:
            return self.__presigned_url
        except AttributeError:
            return None

    @presigned_url.setter
    def presigned_url(self, presigned_url: Optional[str]):
        presigned_url_ = str(presigned_url) if presigned_url is not None else None
        if presigned_url_ is not None and not presigned_url_.startswith('https://'):
            raise ValueError(f'Invalid presigned URL {presigned_url_}')
        self.__presigned_url = presigned_url_

    def set_s3_uri_from_bucket_and_key(self, bucket: Optional[str], key: Optional[str] = None):
        """
        Creates a S3 URI from the given bucket and key, and sets the s3_uri property with it.

        :param bucket: a bucket name. If None, the object's S3 URI will be set to None.
        :param key: a key (optional).
        """
        if bucket is None:
            self.__s3_uri = None
        else:
            self.__s3_uri = _s3_uri(bucket, key)


def _s3_uri(bucket: str, key: Optional[str]) -> str:
    """
    Creates and returns a S3 URI from the given bucket and key.

    :param bucket: a bucket name (required).
    :param key: a key (optional).
    """
    if bucket is None:
        raise ValueError('bucket is required')
    return f"s3://{bucket}/{key if key is not None else ''}"


class S3StorageClassMixin:
    """
    Mixin for adding a storage class property to a desktop object.
    """

    def __init__(self):
        self.__storage_class = S3StorageClass.STANDARD

    @property
    def storage_class(self) -> S3StorageClass:
        """The AWS S3 storage class of this file."""
        try:
            return self.__storage_class
        except AttributeError:
            return S3StorageClass.STANDARD

    @storage_class.setter
    def storage_class(self, storage_class: S3StorageClass):
        if storage_class is None:
            self.__storage_class = S3StorageClass.STANDARD
        elif isinstance(storage_class, S3StorageClass):
            self.__storage_class = storage_class
        else:
            self.__storage_class = S3StorageClass[storage_class]



from heaobject import root


class DataElement(root.AbstractDesktopObject):

    def __init__(self):
        super().__init__()
        self.value_domain = None
        self.concept_uri = None
        self.context_uri = None
        self.workflow_status = None



from .root import AbstractDesktopObject
from typing import Optional
from email_validator import validate_email, EmailNotValidError  # Leave this here for other modules to use
from functools import partial


class Person(AbstractDesktopObject):
    """
    Represents a Person
    """

    def __init__(self):
        super().__init__()
        # id is a super field
        # name is inherited in super
        self.__preferred_name: Optional[str] = None
        self.__first_name: Optional[str] = None
        self.__last_name: Optional[str] = None
        self.__title: Optional[str] = None
        self.__email: Optional[str] = None
        self.__phone_number: Optional[str] = None
        self.__id_labs_manage: Optional[list] = None
        self.__id_labs_member: Optional[list] = None
        self.__id_labs_collaborator: Optional[list] = None
        self.__id_projects_collaborator: Optional[list] = None

    @property
    def preferred_name(self) -> Optional[str]:
        """
        The Person's preferred name (Optional).
        """
        return self.__preferred_name

    @preferred_name.setter
    def preferred_name(self, preferred_name: Optional[str]) -> None:
        self.__preferred_name = str(preferred_name) if preferred_name is not None else None

    @property
    def first_name(self) -> Optional[str]:
        """
        The Person's first name or given name (Optional).
        """
        return self.__first_name

    @first_name.setter
    def first_name(self, first_name: Optional[str]) -> None:
        self.__first_name = str(first_name) if first_name is not None else None

    @property
    def last_name(self) -> Optional[str]:
        """
          The Person's last name (Optional).
        """
        return self.__last_name

    @last_name.setter
    def last_name(self, last_name: Optional[str]) -> None:
        self.__last_name = str(last_name) if last_name is not None else None

    @property
    def title(self) -> Optional[str]:
        """
          The Person's title (Optional).
        """
        return self.__title

    @title.setter
    def title(self, title: Optional[str]) -> None:
        self.__title = str(title) if title is not None else None

    @property
    def email(self) -> Optional[str]:
        """
        The person's email (Optional). Must be a valid e-mail address or None.
        """
        return self.__email

    @email.setter
    def email(self, email: Optional[str]) -> None:
        self.__email = _validate_email(str(email)).email if email is not None else None

    @property
    def phone_number(self) -> Optional[str]:
        """
          The Person's phone number (Optional).
        """
        return self.__phone_number

    @phone_number.setter
    def phone_number(self, phone_number: Optional[str]) -> None:
        self.__phone_number = str(phone_number) if phone_number is not None else None

    @property
    def id_labs_manage(self) -> Optional[list]:
        """
          The Person's list of labs they manage (Optional).
        """
        return self.__id_labs_manage

    @id_labs_manage.setter
    def id_labs_manage(self, id_labs_manage: Optional[list]) -> None:
        if id_labs_manage is None:
            self.__id_labs_manage = None
        else:
            self.__id_labs_manage = [str(item) for item in id_labs_manage if item is not None]

    @property
    def id_labs_member(self) -> Optional[list]:
        """
          The Person's list of labs they are a member of (Optional).
        """
        return self.__id_labs_member

    @id_labs_member.setter
    def id_labs_member(self, id_labs_member: Optional[list]) -> None:
        if id_labs_member is None:
            self.__id_labs_member = None
        else:
            self.__id_labs_member = [str(item) for item in id_labs_member if item is not None]

    @property
    def id_labs_collaborator(self) -> Optional[list]:
        """
          The Person's list of labs they are listed as a collaborator of (Optional).
        """
        return self.__id_labs_collaborator

    @id_labs_collaborator.setter
    def id_labs_collaborator(self, id_labs_collaborator: Optional[list]):
        if id_labs_collaborator is None:
            self.__id_labs_collaborator = None
        else:
            self.__id_labs_collaborator = [str(item) for item in id_labs_collaborator if item is not None]

    @property
    def id_projects_collaborator(self) -> Optional[list]:
        """
          The Person's list of projects they are listed as a collaborator of (Optional).
        """
        return self.__id_projects_collaborator

    @id_projects_collaborator.setter
    def id_projects_collaborator(self, id_projects_collaborator: Optional[list]):
        if id_projects_collaborator is None:
            self.__id_projects_collaborator = None
        else:
            self.__id_projects_collaborator = [str(item) for item in id_projects_collaborator if item is not None]

    @property
    def display_name(self) -> Optional[str]:
        """
        The Person's full name, in First Last order.
        """
        fname = self.first_name if self.first_name else ""
        lname = self.last_name if self.last_name else ""
        s = ' ' if fname and lname else ''
        return f'{fname}{s}{lname}'


_validate_email = partial(validate_email, check_deliverability=False)



import dateutil.parser
from datetime import date
from typing import Optional
from .data import DataObject, SameMimeType
from abc import ABC

from .aws import S3URIMixin, S3StorageClassMixin


class Storage(DataObject, ABC):
    """
    Abstract base class for AWS S3 Storage.
    """
    pass


class AWSStorage(Storage, SameMimeType, S3URIMixin, S3StorageClassMixin):
    """
    Represents an AWS S3 Storage in the HEA desktop. Contains functions that allow access and setting of the value.
    """

    def __init__(self):
        super().__init__()
        self.__arn: Optional[str] = None
        self.__storage_bytes: Optional[int] = None
        self.__min_storage_duration: Optional[int] = None
        self.__object_count: Optional[int] = None
        self.__object_init_modified: Optional[date] = None
        self.__object_last_modified: Optional[date] = None
        self.__volume_id: Optional[str] = None

    @classmethod
    def get_mime_type(cls) -> str:
        """
        Returns the mime type for AWSStorage objects.

        :return: application/x.awsstorage
        """
        return 'application/x.awsstorage'

    @property
    def mime_type(self) -> str:
        """Read-only. The mime type for AWSStorage objects, application/x.awsstorage."""
        return type(self).get_mime_type()

    @property
    def arn(self) -> Optional[str]:
        """Returns the aws arn str for identifying resources on aws"""
        return self.__arn

    @arn.setter
    def arn(self, arn: Optional[str]) -> None:
        """Sets the numerical account identifier"""
        self.__arn = str(arn) if arn is not None else None

    @property
    def storage_bytes(self) -> Optional[int]:
        """Returns the storage bytes"""
        return self.__storage_bytes

    @storage_bytes.setter
    def storage_bytes(self, storage_bytes: int) -> None:
        """Sets the storage bytes"""
        self.__storage_bytes = int(storage_bytes) if storage_bytes is not None else None

    @property
    def min_storage_duration(self) -> Optional[int]:
        """Returns the minimum storage days of the storage class"""
        return self.__min_storage_duration

    @min_storage_duration.setter
    def min_storage_duration(self, min_storage_duration: int) -> None:
        """Sets the minimum storage days of the storage class"""
        self.__min_storage_duration = int(min_storage_duration) if min_storage_duration is not None else None

    @property
    def object_count(self) -> Optional[int]:
        """Returns the total object count of the storage class"""
        return self.__object_count

    @object_count.setter
    def object_count(self, object_count: int) -> None:
        """Sets the total object count of the storage class"""
        self.__object_count = int(object_count) if object_count is not None else None

    @property
    def object_init_modified(self) -> Optional[date]:
        return self.__object_init_modified

    @object_init_modified.setter
    def object_init_modified(self, value: Optional[date | str]) -> None:
        if value is None or isinstance(value, date):
            self.__object_init_modified = value
        else:
            self.__object_init_modified = dateutil.parser.isoparse(value)

    @property
    def object_last_modified(self) -> Optional[date]:
        return self.__object_last_modified

    @object_last_modified.setter
    def object_last_modified(self, value: Optional[date | str]) -> None:
        if value is None or isinstance(value, date):
            self.__object_last_modified = value
        else:
            self.__object_last_modified = dateutil.parser.isoparse(value)

    @property
    def volume_id(self) -> Optional[str]:
        return self.__volume_id

    @volume_id.setter
    def volume_id(self, volume_id: Optional[str]) -> None:
        self.__volume_id = str(volume_id) if volume_id is not None else None



"""
Various utility functions that may be useful throughout heaobject.
"""


def str_to_bool(string: str) -> bool:
    """
    Returns a bool that is appropriate for the given input string. Strings such as "True", "true", "yes", "T", "y",
    and "Y" will return True, and strings such as "False", "false", "no", "t", "n", "N", and "" will return False. If
    string is not recognized, False will be returned.
    """
    return str(string).strip().lower() in ('true', 't', 'yes', 'y')
